function model = warmUpLoopless(model,numWarmup,method,params)
% Performs sample preparation
% Inputs:  model, numWarmup (optional), method ('loopless' or
% 'conventional'), gurobi optimizer parameters
% Outputs: model
%%%%%%%%%%%%%%%%%%%%%% Pedro Saa UQ 2015 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define number of warmup points
if nargin<2
    numWarmup             = 2*model.numRxns;
    method                = 'loopless';
    params.outputflag     = 0;
    params.OptimalityTol  = 1e-9;
    params.IntFeasTol     = 1e-9;
    params.FeasibilityTol = 1e-9;
    params.MIPGapAbs      = 1e-12;
    params.MIPGap         = 1e-12;
elseif nargin<3
    method                = 'loopless';
    params.outputflag     = 0;
    params.OptimalityTol  = 1e-9;
    params.IntFeasTol     = 1e-9;
    params.FeasibilityTol = 1e-9;
    params.MIPGapAbs      = 1e-12;
    params.MIPGap         = 1e-12;
elseif nargin<4
    params.outputflag     = 0;
    params.OptimalityTol  = 1e-9;
    params.IntFeasTol     = 1e-9;
    params.FeasibilityTol = 1e-9;
    params.MIPGapAbs      = 1e-12;
    params.MIPGap         = 1e-12;    
end

% Initialize main variables
progIndicator         = fix(model.numRxns/10);
warmupPoints          = zeros(model.numRxns,numWarmup);
modelLP               = linearStructure(model);
numTol                = 1e-8;
t0                    = cputime;

% Perform loopless warmup
if strcmp(method,'loopless')
    model.obj             = zeros(size(model.obj));
    lbRef                 = model.lb(1:model.numRxns);
    ubRef                 = model.ub(1:model.numRxns);    
    nonZeroElements       = sum((model.Nint~=0),2);
    loopMatrix            = sign(model.Nint);
    alpha                 = 0.95;
    
    % Main loop
    for counter = 1:numWarmup/2
        % Display progress
        if ~mod(counter,progIndicator) && counter>0
            disp(['Current progress... ',num2str(10*counter/progIndicator),'%']);
        end
        
        % Solve maximization problem (try first solving the LP problem)
        modelLP.obj(counter)     = 1;
        modelLP.ub(counter)      = lbRef(counter)+alpha*(ubRef(counter)-lbRef(counter));
        sol                      = gurobi(modelLP,params);
        sol.x(abs(sol.x)<numTol) = 0;   % Remove solutions below numerical tol
        modelLP.ub(counter)      = ubRef(counter);
        
        % Check feasibility        
        checkPattern  = diag(sign(sol.x(model.internal)));
        if isempty(model.Nint) || all(abs(sum(loopMatrix*checkPattern,2))~=nonZeroElements)
            warmupPoints(:,2*counter-1) = sol.x(1:model.numRxns);
            
            % Run MILP problem if solution is unfeasible
        else
            model.obj(counter)          = 1;
            model.ub(counter)           = lbRef(counter)+alpha*(ubRef(counter)-lbRef(counter));
            sol                         = gurobi(model,params);
            sol.x(abs(sol.x)<numTol)    = 0;   % Remove solutions below numerical tol
            warmupPoints(:,2*counter-1) = sol.x(1:model.numRxns);
            model.ub(counter)           = ubRef(counter);            
        end
        
        % Solve minimization problem (try first solving the LP problem)
        modelLP.obj(counter)     = -1;
        modelLP.lb(counter)      = lbRef(counter)+(1-alpha)*(ubRef(counter)-lbRef(counter));
        sol                      = gurobi(modelLP,params);
        sol.x(abs(sol.x)<numTol) = 0;   % Remove solutions below numerical tol
        modelLP.lb(counter)      = lbRef(counter);
        
        % Check feasibility of the LP solution        
        checkPattern  = diag(sign(sol.x(model.internal)));
        if isempty(model.Nint) || all(abs(sum(loopMatrix*checkPattern,2))~=nonZeroElements)
            warmupPoints(:,2*counter) = sol.x(1:model.numRxns);
            
            % Run MILP problem if solution is unfeasible
        else
            model.obj(counter)        = -1;
            model.lb(counter)         = lbRef(counter)+(1-alpha)*(ubRef(counter)-lbRef(counter));
            sol                       = gurobi(model,params);
            sol.x(abs(sol.x)<numTol)  = 0;   % Remove solutions below numerical tol
            warmupPoints(:,2*counter) = sol.x(1:model.numRxns);
            model.lb(counter)         = lbRef(counter);            
        end
        
        % Reset original obj. values
        modelLP.obj(counter) = 0;
        model.obj(counter)   = 0;
    end
    model.warmupMethod = 'loopless';
    
else % Perform conventional warmup as in ACHR
    modelLP = linearStructure(model);
    
    for counter = 1:numWarmup/2
        % Show progress
        if ~mod(counter,progIndicator) && counter>0
            disp(['Current progress... ',num2str(10*counter/progIndicator),'%']);
        end
        
        % Solve maximization problem
        modelLP.obj(counter)        = 1;
        sol                         = gurobi(modelLP,params);
        sol.x(abs(sol.x)<numTol)    = 0;
        warmupPoints(:,2*counter-1) = sol.x;
        
        % Reset previous parameters and solve minimization problem
        modelLP.obj(counter)        = -1;
        sol                         = gurobi(modelLP,params);
        sol.x(abs(sol.x)<numTol)    = 0;
        warmupPoints(:,2*counter)   = sol.x;
        
        % Reset original obj. value
        modelLP.obj(counter) = 0;
    end
    model.warmupMethod = 'conventional';
end

% Extract centroid
model.warmupTime = (cputime-t0)/60;        % FVA warming-up time
model.warmUp     = warmupPoints;
model.centroid   = mean(warmupPoints,2);
model.centroid(abs(model.centroid)<numTol) = 0;   % Remove solutions below numerical tol

% Compute kernel
model.N = null(full(model.S));